# coding: utf-8
import sys
if sys.version_info > (3,0):
    from _tkinter import *
else:
    from Tkinter import *


class Sudoku():
    def __init__(self, root, title):
        self.root = root
        self.initUI(title)
        self.centerWindow()

    def initUI(self, title=u'数　独'):

        self.root.title(title)

        self.label01 = Label(self.root, text=u'問題を左のテーブルに入力し、「開始」ボタンを押してください')
        self.label01.pack()

        self.sudoku_message=StringVar()
        self.sudoku_msg = Label(self.root, textvariable=self.sudoku_message)
        self.sudoku_msg.pack()


        # --------------------------------------------------------------
        sdFr1 = Frame(self.root, relief=FLAT, bd=30)
        sdFr1.pack()
        self.ent, self.kai = [], []
        self.ent_var, self.kai_var = [[StringVar() for i in range(9)] for j in range(9)], [[StringVar() for i in range(9)] for j in range(9)]
        for r in range(9):
            ce, ck, br = [], [], r // 3
            for c in range(9):
                bc = c // 3
                if br == 1:
                    if bc == 1:
                        bcolor = 'lightblue1'
                    else:
                        bcolor = 'white'
                else:
                    if bc == 1:
                        bcolor = 'white'
                    else:
                        bcolor = 'lightblue1'
                ce.append(Entry(sdFr1,justify="center", width=3, bg=bcolor, textvariable=self.ent_var[r][c]))
                ce[-1].grid(row=r, column=c)
                ck.append(Entry(sdFr1,justify="center",  width=3, bg=bcolor, textvariable=self.kai_var[r][c]))
                ck[-1].grid(row=r, column=c+12)

            self.ent.append(ce)
            self.kai.append(ck)



        self.btnStart = Button(sdFr1, text=u'開始', bd=5, command= self.OnBtnStart)
        self.btnStart.grid(row=4,column=10)

        self.btnReset = Button(sdFr1, text=u'問題のリセット', bd=5, command=self.OnBtnReset)
        self.btnReset.grid(row=7, column=10)



        #---------------------------------------------------------------


    def centerWindow(self):
        w, h = 600, 400
        sw = self.root.winfo_screenwidth()
        sh = self.root.winfo_screenheight()
        x, y = (sw - w)//2, (sh - h)//2
        self.root.geometry('%dx%d+%d+%d' % (w, h, x, y))


    def OnBtnStart(self):

        TF,  L, R, C, B = self.check_ent()

        if not TF:
            return
        P, SD, SOL = self.set_data(L, R, C, B)
        self.set_kaiTable(SOL)
        sol = self.solve_it(P, SD, R, C, B, SOL)
        # print(sol)
        # self.set_kaiTable(sol)
        return

    def OnBtnReset(self):
        L=[['' for i in range(9)] for j in range(9)]
        self.set_entTable(L)

    def get_ent(self, r, c):
        return self.ent_var[r][c].get()

    def get_entTable(self):
        L=[[0 for i in range(9)] for j in range(9)]
        for r in range(9):
            for c in range(9):
                L[r][c] = self.ent_var[r][c].get()
        return L

    def set_ent(self, r, c, v):
        self.ent_var[r][c].set(v)

    def set_entTable(self, L):
        for r, l in enumerate(L):
            for c, v in enumerate(l):
                self.ent_var[r][c].set(v)

    def get_kai(self, r, c):
        return self.kai_var[r][c].get()

    def get_kaiTable(self):
        L = [[0 for i in range(9)] for j in range(9)]
        for r in range(9):
            for c in range(9):
                L[r][c] = self.kai_var[r][c].get()
        return L

    def set_kai(self, r, c, v):
        self.kai_var[r][c].set(v)

    def set_kaiTable(self, L):
        for r , l in enumerate(L):
            for c, v in enumerate(l):
                self.kai_var[r][c].set(v)

    def check_ent(self):

        R, C, B = [[] for i in range(9)], [[] for i in range(9)], [[[], [], []], [[], [], []], [[], [], []]]

        L=self.get_entTable()
        mstring=''
        for r,l in enumerate(L):
            for c, v in enumerate(l):
                if v=='':
                    L[r][c] = 0
                elif v.isdigit():
                    n, br, bc = int(v), r // 3, c // 3
                    if 0 < n < 10:
                        if n in R[r] or n in C[c] or n in B[br][bc]:
                            mstring += "\n {0} が重複しています".format(v)
                        else:
                            R[r].append(n)
                            C[c].append(n)
                            B[br][bc].append(n)
                            L[r][c] = n
                    else:
                        mstring += "\n {0} は 1~9 の数字ではありません".format(v)
                else:
                    mstring += "\n {0} は数字 1~9 以外の文字です".format(v)
        if mstring != '':
            self.sudoku_message.set(mstring)
            return (False, L, R, C, B)
        else:
            return (True, L, R, C, B)

    def set_data(self, L, R, C, B):

        p, sd = [0], [ [ [e for e in range(1,10)] for r in range(9)] for c in range(9)]
        sol = [[0 for i in range(9)] for i in range(9)]
        for r, l in enumerate(L):

            for c , n in enumerate(l):
                br, bc = r//3, c//3
                if n == 0:
                    p.append([(r,c, br, bc), 0])
                else:
                    sol[r][c]= n

            for r in range(9):
                for c in range(9):
                    br, bc = r//3, c//3
                    sd[r][c] = list(set(sd[r][c]) - (set(R[r]) | set(C[c]) | set(B[br][bc])))
        return p, sd, sol

    def solve_it(self, P,SD,R,C,B,SOL):

        Pi, Plast = 1, len(P)

        dbPi=0

        while Pi < Plast:
            if Pi<0:
                print ('data error!?')
                return None
            r, c, br, bc = P[Pi][0]
            e = P[Pi][1]
            while e < len(SD[r][c]):
                n = SD[r][c][e]
                if n in R[r] or n in C[c] or n in B[br][bc]:
                    e += 1
                    continue
                else:
                    P[Pi][1] = e
                    SOL[r][c] = n #-------------------------------------------------
                    self.set_kai(r, c, n)
                    R[r].append(n)
                    C[c].append(n)
                    B[br][bc].append(n)
                    Pi += 1
                    break
            else:
                if SOL[r][c] != 0:
                    if SOL[r][c] in R[r]: R[r].remove(SOL[r][c])
                    if SOL[r][c] in C[c]: C[c].remove(SOL[r][c])
                    if SOL[r][c] in B[br][bc]: B[br][bc].remove(SOL[r][c])

                SOL[r][c] = 0 #-------------------------------------------------
                self.set_kai(r, c, 0)
                for i in range(Pi,Plast):
                    P[Pi][1] = 0
                Pi -= 1
                r, c, br, bc = P[Pi][0]
                P[Pi][1] += 1
                if SOL[r][c] in R[r]: R[r].remove(SOL[r][c])
                if SOL[r][c] in C[c]: C[c].remove(SOL[r][c])
                if SOL[r][c] in B[br][bc]: B[br][bc].remove(SOL[r][c])


        else:
            return SOL

def main():
    root = Tk()

    root.lift()
    root.attributes('-topmost', True)
    root.after_idle(root.attributes, '-topmost', False)

    sudo = Sudoku(root, u'数　独   from  梅屋萬年堂')
    root.mainloop()

#----------------------------------------- mainloop
if __name__ == '__main__':
    main()

# # ------------------for debug
# self.ent_var[0][2].set('3')
# self.ent_var[0][4].set('2')
# self.ent_var[0][6].set('6')
# self.ent_var[1][0].set('9')
# self.ent_var[1][3].set('3')
# self.ent_var[1][5].set('5')
# self.ent_var[1][8].set('1')
# self.ent_var[2][2].set('1')
# self.ent_var[2][3].set('8')
# self.ent_var[2][5].set('6')
# self.ent_var[2][6].set('4')
# self.ent_var[3][2].set('8')
# self.ent_var[3][3].set('1')
# self.ent_var[3][5].set('2')
# self.ent_var[3][6].set('9')
# self.ent_var[4][0].set('7')
# self.ent_var[4][8].set('8')
# self.ent_var[5][2].set('6')
# self.ent_var[5][3].set('7')
# self.ent_var[5][5].set('8')
# self.ent_var[5][6].set('2')
# self.ent_var[6][2].set('2')
# self.ent_var[6][3].set('6')
# self.ent_var[6][5].set('9')
# self.ent_var[6][6].set('5')
# self.ent_var[7][0].set('8')
# self.ent_var[7][3].set('2')
# self.ent_var[7][5].set('3')
# self.ent_var[7][8].set('9')
# self.ent_var[8][2].set('5')
# self.ent_var[8][4].set('1')
# self.ent_var[8][6].set('3')
# # ------------------for debug
